;;; create-window-request.scm

(define create-window-set-type
  (make-set-type
   32  ; bit-size
   0 0 0 0 0 0 0 0    0 0 0 0 0 0 0 0    0
                           ; DEFAULT VALUES
   'cursor                 ; none
   'colormap               ; copy-from-parent
   'do-not-propagate-mask  ; ()
   'event-mask             ; ()
   'save-under             ; #f
   'override-redirect      ; #f
   'backing-pixel          ; 0
   'backing-planes         ; all ones
   'backing-store          ; not-useful
   'win-gravity            ; north-west
   'bit-gravity            ; forget
   'border-pixel           ; 0
   'border-pixmap          ; copy-from-parent
   'background-pixel       ; transparent
   'background-pixmap      ; none
   ))


;;; This is ordered ls bit to ms bit

(define create-window-components
  '(background-pixmap background-pixel border-pixmap border-pixel
    bit-gravity win-gravity backing-store backing-planes backing-pixel
    override-redirect save-under event-mask do-not-propagate-mask colormap
    cursor))


(define background-pixmap/fill!
  (lambda (str offset background-pixmap)
    (number->card32/fill!
     str offset
     (case background-pixmap
       ((none)            0)
       ((parent-relative) 1)
       (else background-pixmap)))))

(define border-pixmap/fill!
  (lambda (str offset border-pixmap)
    (number->card32/fill!
     str offset
     (case border-pixmap
       ((copy-from-parent) 0)
       (else border-pixmap)))))

(define backing-store/fill!
  (lambda (str offset backing-store)
    (number->card8/fill!
     str offset
     (case backing-store
       ((not-useful)  0)
       ((when-mapped) 1)
       ((always)      2)
       (else  (error "bad attribute value for backing-store" backing-store))))))

(define colormap/fill!
  (lambda (str offset colormap)
    (number->colormap/fill!
     str offset
     (case colormap
       ((copy-from-parent) 0)
       (else colormap)))))

(define cursor/fill!
  (lambda (str offset cursor)
    (number->cursor/fill!
     str offset
     (case cursor
       ((none) 0)
       (else cursor)))))

(define process-create-window-attributes-alist
  (lambda (alist)
    (let ((set (let ((proto-set (make-set create-window-set-type 0)))
		 (for-each (lambda (cell)
			     (set-bit proto-set (car cell)))
			   alist)
		 proto-set))
	  (bit-count (length alist)))
      ;; value-list size + bit-mask size
      (let ((str (make-string (+ (* 4 bit-count) 4) null-char)))
	(number->card32/fill! str 0 (set->number set))
	(let loop ((components create-window-components) (offset 4))
	  (cond ((null? components)
		 (values str bit-count))
		((assq (car components) alist) =>
		 (lambda (component-cell)
		   (let ((component (car component-cell))
			 (value     (cadr component-cell)))
		     (case component
		       ((background-pixmap) (background-pixmap/fill! str offset value))
		       ((background-pixel) (number->card32/fill! str offset value))
		       ((border-pixmap) (border-pixmap/fill!  str offset value))
		       ((border-pixel)  (number->card32/fill! str offset value))
		       ((bit-gravity) (fill-string! str offset (symbol->bit-gravity value)))
		       ((win-gravity) (fill-string! str offset (symbol->win-gravity value)))
		       ((backing-store)  (backing-store/fill!  str offset value))
		       ((backing-planes) (number->card32/fill! str offset value))
		       ((backing-pixel)  (number->card32/fill! str offset value))
		       ((override-redirect) (boolean->bool/fill! str offset value))
		       ((save-under) (boolean->bool/fill! str offset value))
		       ((event-mask) (symbols->set-of-event/fill! str offset value))
		       ((do-not-propagate-mask)
			(symbols->set-of-device-event/fill! str offset value))
		       ((colormap) (colormap/fill! str offset value))
		       ((cursor)   (cursor/fill! str offset value))
		       (else (error "unrecognized component" component)))
		     ;; value-list is a tetrabyte array
		     (loop (cdr components) (+ offset 4)))))
		(else (loop (cdr components) offset))))))))


;;;;;;;;;;;;;;;;;;;;;;;;
;;; CreateWindow     ;;;
;;; xp-create-window ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

(define create-window
  (let ((opcode 1)
	(lock (make-lock))
	(str  (make-string 28)))
    (number->card8/fill! str 0 opcode)
    (lambda (conn parent-window x y width height border-width depth
	     window-class visual attributes-alist)
      (bind ((bitmask&value-list n (process-create-window-attributes-alist attributes-alist)))
	(let ((xserver (connection-xserver conn))
	      (window-id  (new-id))
	      (v-id (case visual
		      ((copy-from-parent) 0)
		      (else visual)))
	      (win-class (case window-class
			   ((copy-from-parent) 0)
			   ((input-output)     1)
			   ((input-only)       2)
			   (else (error "illegal window-class" window-class))))
	      (request-length (+ 8 n)))
	  (with-lock lock
            (lambda ()
	      (number->depth/fill!  str  1 depth)
	      (number->card16/fill! str  2 request-length)
	      (number->window/fill! str  4 window-id)
	      (number->window/fill! str  8 parent-window)
	      (number->int16/fill!  str 12 x)
	      (number->int16/fill!  str 14 y)
	      (number->card16/fill! str 16 width)
	      (number->card16/fill! str 18 height)
	      (number->card16/fill! str 20 border-width)
	      (number->card16/fill! str 22 win-class)
	      (number->visual-id/fill! str 24 v-id)
	      (send xserver str bitmask&value-list)
	      window-id)))))))
