;;; helpers.scm

(define opt->screen-num
  (lambda (opt)
    (if (null? opt)
	0
	(let ((screen (car opt)))
	  (if (and (integer? screen)
		   (>= screen 1))
	      screen
	      (error "A screen number should be an integer >= 1"))))))

(define connection-screen
  (lambda (connection . opt-screen-num)
    (let ((screens      (connection:root-screens-vector connection))
	  (screen-index (opt->screen-num opt-screen-num)))
      (let ((number-of-screens (vector-length screens)))
	(if (>= screen-index number-of-screens)
	    (error "You're asking for a screen that isn't there"
		   screen-index number-of-screens)
	    (vector-ref screens screen-index))))))

(define root-window
  (lambda (connection . opt)
    (let ((screen (apply connection-screen (cons connection opt))))
      (screen:root screen))))

(define default-colormap
  (lambda (connection . opt)
    (let ((screen (apply connection-screen (cons connection opt))))
      (screen:default-colormap screen))))

(define root-depth
  (lambda (connection . opt)
    (let ((screen (apply connection-screen (cons connection opt))))
      (screen:root-depth screen))))

(define root-visual
  (lambda (connection . opt)
    (let ((screen (apply connection-screen (cons connection opt))))
      (screen:root-visual screen))))

(define white-pixel
  (lambda (connection . opt)
    (let ((screen (apply connection-screen (cons connection opt))))
      (screen:white-pixel screen))))

(define black-pixel
  (lambda (connection . opt)
    (let ((screen (apply connection-screen (cons connection opt))))
      (screen:black-pixel screen))))

(define-syntax with-connection
  (syntax-rules ()
    ((with-connection func1)
     (let ((connection #f))
       (dynamic-wind
	   (lambda () (set! connection (make-connection)))
	   (lambda ()
	     (func1 connection))
	   (lambda ()
	     (close-connection connection)))))))

(define xserver-read-event read-event)

(define read-event
  (lambda (conn)
    (xserver-read-event (connection-xserver conn))))
