;;; odd-font.scm

(define *mag* 1)

(define previewer
  (lambda (mag filename)
    (with-connection
     (lambda (connection)
       (set! *mag* mag)
       (let ((root    (root-window connection))
	     (depth   (root-depth connection))
	     (white   (white-pixel connection))
	     (black   (black-pixel connection))
             (line-height 8)
             (line-width 1)
             (nr-chars-wide 120)
             (nr-chars-high 25))
	 (let* ((window (create-window connection root
                                       0 0
                                       (* *mag* nr-chars-wide line-width)
                                       (* *mag* (+ (* (- nr-chars-high 1) (+ line-height 1))
                                                   line-height))
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure button-press))
					 (background-pixel ,white))))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white)) )))
	   (format #t "Mapping Window~%")
	   (map-window connection window)
           
	   (let loop ((event (read-event connection)))
	     (format #t "The event we get: ~s~%" event)
	     (cond ((button-press-event? event) #t)
                   ((expose-event? event)
                    (with-input-from-file filename
                      (lambda ()
                        (do ((yline 0 (+ yline 1))
                             (line (read-line) (read-line)))
                            ((or (>= yline nr-chars-high)
                                 (eof-object? line)) #t)
                          (let ((y (* line-height yline *mag*))
                                (len (string-length line)))
                            (do ((x 0 (+ x *mag*))
                                 (i 0 (+ i 1)))
                                ((>= i len) #t)
                              (format #t "~a ~a ~a~%" x i len)
                              (let ((c (string-ref line i)))
                                (when (char=? c (ascii->char 9))
                                  (set! line (string-append "       " line))
                                  (set! len (+ 8 len))
                                  (set! c #\space))
                                (draw-char connection window gc x y c)))))))
                    (loop (read-event connection)))
                   (else
                    (loop (read-event connection)))))))))))

(define char->bits
  (lambda (char)
    (let ((int (char->ascii char)))
      (let ((bits (let loop ((i 7) (num int) (bits '()))
                    (cond ((zero? i) bits)
                          ((odd? num) (loop (- i 1)
                                            (arithmetic-shift num -1)
                                            (cons #t bits)))
                          ((even? num) (loop (- i 1)
                                             (arithmetic-shift num -1)
                                             (cons #f bits)))))))
        (reverse bits)))))

(define draw-char
  (lambda (connection drawable gc x y char)
    (let* ((bits (char->bits char))
           (rectangles (let loop ((y y) (bits bits) (recs '()))
                         (cond ((null? bits)
                                (reverse recs))
                               ((eq? (car bits) #t)
                                (loop (+ y *mag*) (cdr bits)
                                      (cons (list x y *mag* *mag*) recs)))
                               ((eq? (car bits) #f)
                                (loop (+ y *mag*) (cdr bits) recs))
                               (else (error "bad input"))))))
      (poly-fill-rectangle connection drawable gc rectangles))))

(define oftest
  (lambda ()
    (previewer 4 "odd-font.scm")))


(define iota
  (lambda (start end . step)
    (let ((step (if (null? step)
                    1
                    (car step))))
      (let loop ((i start) (nums '()))
        (if (>= i end)
            (reverse nums)
            (loop (+ i step) (cons i nums)))))))

(define string-for-each
  (lambda (f str)
    (let ((len (string-length str)))
      (for-each (lambda (i)
                  (f (string-ref str i)))
                (iota 0 len)))))
