;;; replies.scm

;;;;;;;;;;;;;;;;;;;;;;;;
;;; AllocColor Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

(define-record alloc-color-reply
  sequence-number
  red
  green
  blue
  pixel)

(define read-alloc-color-reply
  (lambda (xserver)
    (let* ((reply-opcode    (read-card8 xserver))   ; ignore, always 1
	   (unused          (ignore 1 xserver))
	   (sequence-number (read-card16 xserver))
	   (length          (read-card32 xserver))  ; always 0
	   (red             (read-card16 xserver))
	   (green           (read-card16 xserver))
	   (blue            (read-card16 xserver))
	   (unused2         (ignore 2 xserver))
	   (pixel           (read-card32 xserver))
	   (unused3         (ignore 12 xserver)))
      (make-alloc-color-reply
       sequence-number red green blue pixel))))

(define display-alloc-color-reply
  (lambda (reply)
    (format #t (string-append "ALLOC-COLOR-REPLY~%"
			      "  seq-num: ~a~%"
			      "  red:     ~a~%"
			      "  green:   ~a~%"
			      "  blue:    ~a~%"
			      "  pixel:   ~a~%")
	    (alloc-color-reply:sequence-number reply)
	    (alloc-color-reply:red reply)
	    (alloc-color-reply:green reply)
	    (alloc-color-reply:blue reply)
	    (alloc-color-reply:pixel reply))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; AllocColorCells Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-record alloc-color-cells-reply
  sequence-number
  pixels
  masks
  )

(define read-alloc-color-cells-reply
  (lambda (xserver)
    (let* ((reply-opcode     (read-card8 xserver))  ; always 1
	   (unused           (read-card8 xserver))
	   (sequence-number  (read-card16 xserver))
	   (reply-length     (read-card32 xserver)) ; m + n
	   (number-of-pixels (read-card16 xserver)) ; n
	   (number-of-masks  (read-card16 xserver)) ; m
	   (unused           (ignore 20 xserver))
	   (pixels           (gather number-of-pixels
				     (lambda () (read-card32 xserver))))
	   (masks            (gather number-of-masks
				     (lambda () (read-card32 xserver)))))
      (make-alloc-color-cells-reply sequence-number pixels masks))))

(define display-alloc-color-cells-reply
  (lambda (reply)
    (format #t (string-append "ALLOC-COLOR-CELLS-REPLY~%"
			      "  sequence-number: ~a~%"
			      "  pixels:          ~a~%"
			      "  masks:           ~a~%")
	    (alloc-color-cells-reply:sequence-number reply)
	    (alloc-color-cells-reply:pixels reply)
	    (alloc-color-cells-reply:masks reply))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; AllocColorPlanes Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-record alloc-color-planes-reply
  sequence-number
  red-mask
  green-mask
  blue-mask
  pixels)

(define read-alloc-color-planes-reply
  (lambda (xserver)
    (let* ((reply-opcode     (read-card8 xserver))  ; always 1
	   (unused           (read-card8 xserver))
	   (sequence-number  (read-card16 xserver))
	   (reply-length     (read-card32 xserver))  ; n
	   (number-of-pixels (read-card16 xserver))  ; also n
	   (unused           (read-card16 xserver))
	   (red-mask         (read-card32 xserver))
	   (green-mask       (read-card32 xserver))
	   (blue-mask        (read-card32 xserver))
	   (unused           (read-card32 xserver))
	   (pixels           (gather number-of-pixels
				     (lambda () (read-card32 xserver)))))
      (make-alloc-color-planes-reply sequence-number red-mask green-mask blue-mask pixels))))

(define display-alloc-color-planes-reply
  (lambda (reply)
    (format #t (string-append "ALLOC-COLOR-PLANES-REPLY~%"
			      "  sequence-number: ~a~%"
			      "  red-mask:        ~a~%"
			      "  green-mask:      ~a~%"
			      "  blue-mask:       ~a~%"
			      "  pixels:          ~a~%")
	    (alloc-color-planes-reply:sequence-number reply)
	    (alloc-color-planes-reply:red-mask reply)
	    (alloc-color-planes-reply:green-mask reply)
	    (alloc-color-planes-reply:blue-mask reply)
	    (alloc-color-planes-reply:pixels reply))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; AllocNamedColor Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-record alloc-named-color-reply
  sequence-number
  pixel
  exact-red
  exact-green
  exact-blue
  visual-red
  visual-green
  visual-blue)

(define read-alloc-named-color-reply
  (lambda (xserver)
    (let* ((reply-opcode    (read-card8 xserver))  ; always one
	   (unused          (read-card8 xserver))
	   (sequence-number (read-card16 xserver))
	   (reply-length    (read-card32 xserver))  ; always zero
	   (pixel           (read-card32 xserver))
	   (exact-red       (read-card16 xserver))
	   (exact-green     (read-card16 xserver))
	   (exact-blue      (read-card16 xserver))
	   (visual-red      (read-card16 xserver))
	   (visual-green    (read-card16 xserver))
	   (visual-blue     (read-card16 xserver))
	   (unused          (ignore 8 xserver)))
      (make-alloc-named-color-reply sequence-number pixel
				    exact-red exact-green exact-blue
				    visual-red visual-green visual-blue))))

(define display-alloc-named-color-reply
  (lambda (reply)
    (format #t (string-append "ALLOC-NAMED-COLOR-REPLY~%"
			      "sequence-number: ~a~%"
			      "  pixel:         ~a~%"
			      "  exact-red:     ~a~%"
			      "  exact-green:   ~a~%"
			      "  exact-blue:    ~a~%"
			      "  visual-red:    ~a~%"
			      "  visual-green:  ~a~%"
			      "  visual-blue:   ~a~%")
	    (alloc-named-color-reply:sequence-number reply)
	    (alloc-named-color-reply:pixel reply)
	    (alloc-named-color-reply:exact-red reply)
	    (alloc-named-color-reply:exact-green reply)
	    (alloc-named-color-reply:exact-blue reply)
	    (alloc-named-color-reply:visual-red reply)
	    (alloc-named-color-reply:visual-green reply)
	    (alloc-named-color-reply:visual-blue reply))))


;;;;;;;;;;;;;;;;;;;;;;
;;; GetImage Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;

(define-record get-image-reply
  depth
  sequence-number
  visual
  data)

(define read-get-image-reply
  (lambda (xserver width*height)
    (let* ((reply-opcode    (read-card8 xserver)) ; always 1
	   (depth           (read-card8 xserver))
	   (sequence-number (read-card16 xserver))
	   ;; reply-length = (n + p) / 4
	   (reply-length    (* 4 (read-card32 xserver)))
	   (visual-id       (let ((visual (read-visual-id xserver)))
			      (if (zero? visual) 'none visual)))
	   (unused          (ignore 20 xserver))
	   (data            (read-list-of-bytes xserver width*height))
	   (padding         (ignore (- reply-length width*height) xserver)))
      (make-get-image-reply depth sequence-number visual-id data))))

(define display-get-image-reply
  (lambda (reply)
    (format #t (string-append "GET-IMAGE-REPLY~%"
			      "  sequence-number: ~a~%"
			      "  depth:           ~a~%"
			      "  visual:          ~a~%"
			      "  data:            ~a~%")
	    (get-image-reply:sequence-number reply)
	    (get-image-reply:depth reply)
	    (get-image-reply:visual reply)
	    (get-image-reply:data reply))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; GetKeyboardMapping Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-record get-keyboard-mapping-reply
  keysyms-per-keycode
  sequence-number
  keysyms)

(define read-get-keyboard-mapping-reply
  (lambda (xserver)
    (let* ((reply-opcode        (read-card8 xserver))  ; always 1
	   (keysyms-per-keycode (read-card8 xserver))
	   (sequence-number     (read-card16 xserver))
	   (reply-length        (read-card32 xserver))
	   (unused              (ignore 24 xserver))
	   (keysyms             (gather reply-length (lambda () (read-keysym xserver)))))
      (make-get-keyboard-mapping-reply
       keysyms-per-keycode
       sequence-number
       keysyms))))

(define display-get-keyboard-mapping-reply
  (lambda (reply)
    (format #t (string-append
		"GET-KEYBOARD-MAPPING REPLY~%"
		"  sequence-number:      ~a~%"
		"  keysyms-per-keycode:  ~a~%"
		"  keysyms:              ~a~%")
	    (get-keyboard-mapping-reply:sequence-number reply)
	    (get-keyboard-mapping-reply:keysyms-per-keycode reply)
	    (get-keyboard-mapping-reply:keysyms reply))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; GetScreenSaver Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-record get-screen-saver-reply
  sequence-number
  timeout
  interval
  prefer-blanking
  allow-exposures)

(define read-get-screen-saver-reply
  (lambda (xserver)
    (let* ((reply-opcode    (read-card8 xserver))   ; ignore, always 1
	   (unused          (ignore 1 xserver))
	   (sequence-number (read-card16 xserver))
	   (length          (read-card32 xserver))  ; always 0
	   (timeout         (read-card16 xserver))
	   (interval        (read-card16 xserver))
	   (prefer-blanking (read-bool xserver))
	   (allow-exposures (read-bool xserver))
	   (unused          (ignore 18 xserver)))
      (make-get-screen-saver-reply sequence-number
				   timeout
				   interval
				   prefer-blanking
				   allow-exposures
				   ))))

(define display-get-screen-saver-reply
  (lambda (reply)
    (format #t (string-append "GET-SCREEN-SAVER-REPLY~%"
			      "  sequence-number: ~a~%"
			      "  timeout:         ~a~%"
			      "  interval:        ~a~%"
			      "  prefer-blanking: ~a~%"
			      "  allow-exposures: ~a~%")
	    (get-screen-saver-reply:sequence-number reply)
	    (get-screen-saver-reply:timeout reply)
	    (get-screen-saver-reply:interval reply)
	    (get-screen-saver-reply:prefer-blanking reply)
	    (get-screen-saver-reply:allow-exposures reply)
	    )))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; ListExtensions Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-record list-extensions-reply
  sequence-number
  names)

(define read-list-extensions-reply
  (lambda (xserver)
    (let* ((reply-opcode    (read-card8 xserver)) ; ignore, always 1
	   (n               (read-card8 xserver)) ; number of strings
	   (sequence-number (read-card16 xserver))
	   (reply-length    (read-card32 xserver)) ; (n + p) / 4
	   (unused          (ignore 24 xserver))
	   (names           (gather n (lambda ()
					(let ((len (read-card8 xserver)))
					  (read-string8 len xserver))))))
      (make-list-extensions-reply sequence-number names))))

(define display-list-extensions-reply
  (lambda (reply)
    (format #t (string-append "LIST-EXTENSIONS-REPLY~%"
			      "  sequence-number: ~a~%"
			      "  names:           ~a~%")
	    (list-extensions-reply:sequence-number reply)
	    (list-extensions-reply:names reply))))


;;;;;;;;;;;;;;;;;;;;;;;
;;; QueryFont Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;

(define-record query-font-reply
  sequence-number
  min-bounds
  max-bounds
  min-char-or-byte2
  max-char-or-byte2
  default-char
  draw-direction
  min-byte1
  max-byte1
  all-chars-exist
  font-ascent
  font-descent
  properties
  char-infos)

(define-record char-info
  left-side-bearing
  right-side-bearing
  character-width
  ascent
  descent
  attributes)

(define read-char-info
  (lambda (xserver)
    (let* ((left-side-bearing  (read-int16 xserver))
	   (right-side-bearing (read-int16 xserver))
	   (character-width    (read-int16 xserver))
	   (ascent             (read-int16 xserver))
	   (descent            (read-int16 xserver))
	   (attributes         (read-card16 xserver)))  ; server dependent
      (make-char-info left-side-bearing
		      right-side-bearing
		      character-width
		      ascent
		      descent
		      attributes))))

(define format-char-info
  (lambda (port char-info)
    (format port (string-append "CHAR-INFO>  left-side-bearing: ~a    right-side-bearing: ~a~%"
				"            ascent: ~a               descent: ~a~%"
				"            character-width: ~a      attrs: ~a~%")
	    (char-info:left-side-bearing char-info)
	    (char-info:right-side-bearing char-info)
	    (char-info:character-width char-info)
	    (char-info:ascent char-info)
	    (char-info:descent char-info)
	    (char-info:attributes char-info))))

(define display-char-info
  (lambda (char-info)
    (format-char-info #t char-info)
    (newline)))

(define read-fontprop
  (lambda (xserver)
    (let* ((atom  (read-atom xserver))
	   (value (read-card32 xserver)))  ; value meaning is based on the property
      (cons atom value))))

(define display-fontprop
  (lambda (fontprop)
    (format #t "FONTPROP  (~a ~a)~%"
	    (car fontprop)
	    (cdr fontprop))))

(define read-query-font-reply
  (lambda (xserver)
    (let* ((reply-opcode      (read-card8 xserver))   ; ignore, always 1
	   (unused            (ignore 1 xserver))
	   (sequence-number   (read-card16 xserver))
	   (length            (read-card32 xserver))
	   (min-bounds        (read-char-info xserver))
	   (unused            (ignore 4 xserver))
	   (max-bounds        (read-char-info xserver))
	   (unused            (ignore 4 xserver))
	   (min-char-or-byte2 (read-card16 xserver))
	   (max-char-or-byte2 (read-card16 xserver))
	   (default-char      (read-card16 xserver))
	   (n                 (read-card16 xserver))   ; number of fontprops
	   (draw-direction    (read-card8 xserver))
	   (min-byte1         (read-card8 xserver))
	   (max-byte1         (read-card8 xserver))
	   (all-chars-exist   (read-bool xserver))
	   (font-ascent       (read-int16 xserver))
	   (font-descent      (read-int16 xserver))
	   (m                 (read-card32 xserver))   ; number of char-infos
	   (properties        (gather n (lambda () (read-fontprop xserver))))
	   (char-infos        (gather m (lambda () (read-char-info xserver)))))
      (make-query-font-reply sequence-number
			     min-bounds
			     max-bounds
			     min-char-or-byte2
			     max-char-or-byte2
			     default-char
			     (case draw-direction
			       ((0) 'left-to-right)
			       ((1) 'right-to-left)
			       (else (error "received odd value for draw-direction"
					    draw-direction)))
			     min-byte1
			     max-byte1
			     all-chars-exist
			     font-ascent
			     font-descent
			     properties
			     char-infos))))

(define display-query-font-reply
  (lambda (reply)
    (format #t (string-append "QUERY-FONT-REPLY~%"
			      "  sequence-number:   ~a~%"
			      "  min-bounds:~%"
			      "~a~%"
			      "  max-bounds:~%"
			      "~a~%"
			      "  min-char-or-byte2: ~a~%"
			      "  max-char-or-byte2: ~a~%"
			      "  default-char:      ~a~%"
			      "  draw-direction:    ~a~%"
			      "  min-byte1:         ~a~%"
			      "  max-byte1:         ~a~%"
			      "  all-chars-exist:   ~a~%"
			      "  font-ascent:       ~a~%"
			      "  font-descent:      ~a~%")
	    (query-font-reply:sequence-number reply)
	    (format-char-info #f (query-font-reply:min-bounds reply))
	    (format-char-info #f (query-font-reply:max-bounds reply))
	    (query-font-reply:min-char-or-byte2 reply)
	    (query-font-reply:max-char-or-byte2 reply)
	    (query-font-reply:default-char reply)
	    (query-font-reply:draw-direction reply)
	    (query-font-reply:min-byte1 reply)
	    (query-font-reply:max-byte1 reply)
	    (query-font-reply:all-chars-exist reply)
	    (query-font-reply:font-ascent reply)
	    (query-font-reply:font-descent reply))
    (let* ((fps (query-font-reply:properties reply))
	   (len (length fps)))
      (if (> len 4)
	  (begin (display-fontprop (car fps))
		 (display-fontprop (cadr fps))
		 (format #t "+ ~a more font properties~%" (- len 2)))
	  (for-each display-fontprop fps)))
    (let* ((infos (query-font-reply:char-infos reply))
	   (len   (length infos)))
      (if (> len 4)
	  (begin (display-char-info (car infos))
		 (display-char-info (cadr infos))
		 (format #t "+ ~a more char-info entries~%" (- len 2)))
	  (for-each display-char-info infos)))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; QueryTextExtents Reply ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define-record query-text-extents-reply
  draw-direction    ; left-to-right or right-to-left
  sequence-number
  font-ascent
  font-descent
  overall-ascent
  overall-descent
  overall-width
  overall-left
  overall-right)

(define read-query-text-extents-reply
  (lambda (xserver)
    (let* ((reply-opcode    (read-card8 xserver))   ; ignore, always 1
	   (draw-direction  (read-card8 xserver))
	   (sequence-number (read-card16 xserver))
	   (length          (read-card32 xserver))  ; always 0
	   (font-ascent     (read-int16 xserver))
	   (font-descent    (read-int16 xserver))
	   (overall-ascent  (read-int16 xserver))
	   (overall-descent (read-int16 xserver))
	   (overall-width   (read-int32 xserver))
	   (overall-left    (read-int32 xserver))
	   (overall-right   (read-int32 xserver))
	   (unused          (ignore 4 xserver)))
      (make-query-text-extents-reply
       (case draw-direction
	 ((0) 'left-to-right)
	 ((1) 'right-to-left)
	 (else (error "odd value received for draw-direction" draw-direction)))
       sequence-number
       font-ascent
       font-descent
       overall-ascent
       overall-descent
       overall-width
       overall-left
       overall-right))))

(define display-query-text-extents-reply
  (lambda (reply)
    (format #t (string-append "QUERY-TEXT-EXTENTS-REPLY~%"
			      "  draw-direction:  ~a~%"
			      "  sequence-number: ~a~%"
			      "  font-ascent:     ~a~%"
			      "  font-descent:    ~a~%"
			      "  overall-ascent:  ~a~%"
			      "  overall-descent: ~a~%"
			      "  overall-width:   ~a~%"
			      "  overall-left:    ~a~%"
			      "  overall-right:   ~a~%")
	    (query-text-extents-reply:draw-direction reply)
	    (query-text-extents-reply:sequence-number reply)
	    (query-text-extents-reply:font-ascent reply)
	    (query-text-extents-reply:font-descent reply)
	    (query-text-extents-reply:overall-ascent reply)
	    (query-text-extents-reply:overall-descent reply)
	    (query-text-extents-reply:overall-width reply)
	    (query-text-extents-reply:overall-left reply)
	    (query-text-extents-reply:overall-right reply)
	    )))
