;;; requests.scm


;;;;;;;;;;;;;;;;;;;
;;; AllocColor  ;;;
;;; alloc-color ;;;
;;;;;;;;;;;;;;;;;;;

(define alloc-color
  (let ((opcode 84)
	(lock (make-lock))
	(str (make-string 16)))
    (number->card8/fill!  str  0 opcode)
    (number->card8/fill!  str  1 0)  ; unused
    (number->card16/fill! str  2 4)  ; request-length
    (number->card16/fill! str 14 0)  ; unused
    (lambda (connection colormap red green blue)
      (with-lock lock
        (lambda ()
	  (number->colormap/fill! str  4 colormap)
	  (number->card16/fill! str  8 red)
	  (number->card16/fill! str 10 green)
	  (number->card16/fill! str 12 blue)
	  (send+read-reply (connection-xserver connection) str opcode))))))


;;;;;;;;;;;;;;;;;;;;;;;;;
;;; AllocColorCells   ;;;
;;; alloc-color-cells ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;

(define alloc-color-cells
  (let ((opcode 86)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill!  str 0 opcode)
    (number->card16/fill! str 2 3)  ; request-length
    (lambda (connection colormap number-of-colors number-of-planes contiguous?)
      (cond ((not (positive? number-of-colors))
	     (error "number-of-colors must be positive" number-of-colors))
	    ((negative? number-of-planes)
	     (error "number-of-planes must be non-negative" number-of-planes))
	    (else
	     (with-lock lock
	       (lambda ()
		 (boolean->bool/fill!    str  1 contiguous?)
		 (number->colormap/fill! str  4 colormap)
		 (number->card16/fill!   str  8 number-of-colors)
		 (number->card16/fill!   str 10 number-of-planes)
		 (send+read-reply (connection-xserver connection) str opcode))))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; AllocColorPlanes   ;;;
;;; alloc-color-planes ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

(define alloc-color-planes
  (let ((opcode 87)
	(lock (make-lock))
	(str (make-string 16)))
    (number->card8/fill! str 0 opcode)
    (number->card8/fill! str 2 4)  ; request length
    (lambda (connection colormap number-of-colors
	     number-of-reds number-of-greens number-of-blues contiguous?)
      (cond ((not (positive? number-of-colors))
	     (error "number-of-colors must be positive" number-of-colors))
	    (else
	     (with-lock lock
	       (lambda ()
		 (boolean->bool/fill!  str  1 contiguous?)
		 (number->colormap/fill! str 4 colormap)
		 (number->card16/fill! str  8 number-of-colors)
		 (number->card16/fill! str 10 number-of-reds)
		 (number->card16/fill! str 12 number-of-greens)
		 (number->card16/fill! str 14 number-of-blues)
		 (send+read-reply (connection-xserver connection) str opcode))))))))


;;;;;;;;;;;;;;;;;;;;;;;;;
;;; AllocNamedColor   ;;;
;;; alloc-named-color ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;

(define alloc-named-color
  (let ((opcode 85)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill!  str  0 opcode)
    (number->card8/fill!  str  1 0)  ; unused
    (number->card16/fill! str 10 0)  ; unused
    (lambda (connection colormap name)
      (bind ((name-length pad-length padded-name (pad-data name)))
	(let ((len (+ 3 (quotient (+ name-length pad-length) 4))))
	  (with-lock lock
            (lambda ()
	      (number->card16/fill!   str 2 len)
	      (number->colormap/fill! str 4 colormap)
	      (number->card16/fill!   str 8 name-length)
	      (send+read-reply (connection-xserver connection)
			       (list str padded-name)
			       opcode))))))))


;;;;;;;;;;;;;;;;;;;;
;;; AllowEvents  ;;;
;;; allow-events ;;;
;;;;;;;;;;;;;;;;;;;;

(define allow-events
  (let ((opcode 35)
	(lock (make-lock))
	(str (make-string 8)))
    (number->card8/fill!  str 0 opcode)
    (number->card16/fill! str 2 2)  ; request length
    (lambda (connection mode time)
      (let ((mode (case mode
		    ((async-pointer)   0)
		    ((sync-pointer)    1)
		    ((replay-pointer)  2)
		    ((async-keyboard)  3)
		    ((sync-keyboard)   4)
		    ((replay-keyboard) 5)
		    ((async-both)      6)
		    ((sync-both)       7)
		    (else (error "unrecognized mode" mode))))
	    (time (case time
		    ((current-time) 0)
		    (else time))))
	(with-lock lock
          (lambda ()
	    (number->card8/fill!     str 1 mode)
	    (number->timestamp/fill! str 4 time)
	    (send (connection-xserver connection) str)))))))


;;;;;;;;;;;;
;;; Bell ;;;
;;; bell ;;;
;;;;;;;;;;;;

(define bell
  (let ((opcode 104)
	(lock (make-lock))
	(str (make-string 4)))
    (number->card8/fill!  str 0 opcode)
    (number->card16/fill! str 2 1)  ; request length
    (lambda (connection percent)
      (if (not (and (integer? percent) (<= -100 percent 100)))
	  (error "argument should be an integer between -100 and 100" percent)
	  (with-lock lock
	    (lambda ()
	      (number->int8/fill! str 1 percent)
	      (send (connection-xserver connection) str)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; ChangeActivePointerGrab    ;;;
;;; change-active-pointer-grab ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define change-active-pointer-grab
  (let ((opcode 30)
	(lock (make-lock))
	(str (make-string 18)))
    (number->card8/fill!  str  0 opcode)
    (number->card8/fill!  str  1 0)  ; unused
    (number->card16/fill! str  2 4)  ; request length
    (number->card16/fill! str 16 0)  ; unused
    (lambda (connection pointer-event-symbols cursor time)
      (let ((cursor (case cursor
		      ((none) 0)
		      (else cursor)))
	    (time (case time
		    ((current-time) 0)
		    (else time))))
	(with-lock lock
	  (lambda ()
	    (number->cursor/fill! str 4 cursor)
	    (number->cursor/fill! str 8 time)
	    (symbols->set-of-pointer-event/fill! str 12 pointer-event-symbols)
	    (send (connection-xserver connection) str)))))))


;;;;;;;;;;;;;;;;
;;; ChangeGC ;;;
;;;;;;;;;;;;;;;;

;;; see create-gc-request.scm


;;;;;;;;;;;;;;;;;;;;
;;; ChangeHosts  ;;;
;;; change-hosts ;;;
;;;;;;;;;;;;;;;;;;;;

(define change-hosts
  (let ((opcode 109)
	(lock (make-lock))
	(str (make-string 8)))
    (number->card8/fill! str 0 opcode)
    (number->card8/fill! str 5 0)  ; unused
    (lambda (connection mode host-family host-address)
      (let ((mode (case mode
		    ((insert) 0)
		    ((delete) 1)
		    (else (error "unrecognized mode" mode))))
	    (family (case host-family
		      ((internet) 0)
		      ;; don't even try ...
		      ((dec-net) (error "DECnet unsupported")) ; 1
		      ((chaos)   (error "Chaos unsupported"))  ; 2
		      (else (error "unsupported host family" host-family))))
	    (address (case host-family
		       ((internet) (internet-address->byte-sequence host-address))
		       ;; let family clause catch errors
		       )))
	(bind ((n p padded-data (pad-data address)))
	  (let ((request-length (+ 2 (/ (+ n p) 4))))
	    (with-lock lock
	      (lambda ()
		(number->card8/fill!  str 1 mode)
		(number->card16/fill! str 2 request-length)
		(number->card8/fill!  str 4 family)
		(number->card16/fill! str 6 n)
		(send (connection-xserver connection) str padded-data)))))))))


;;;;;;;;;;;;;;;;;;
;;; ClearArea  ;;;
;;; clear-area ;;;
;;;;;;;;;;;;;;;;;;

(define clear-area
  (let ((opcode 61)
	(lock (make-lock))
	(str (make-string 16)))
    (number->card8/fill!  str 0 opcode)
    (number->card16/fill! str 2 4)  ; request length
    (lambda (connection window x y width height exposures)
      (with-lock lock
        (lambda ()
	  (boolean->bool/fill!  str  1 exposures)
	  (number->window/fill! str  4 window)
	  (number->int16/fill!  str  8 x)
	  (number->int16/fill!  str 10 y)
	  (number->card16/fill! str 12 width)
	  (number->card16/fill! str 14 height)
	  (send (connection-xserver connection) str))))))


;;;;;;;;;;;;;;;;;
;;; CopyArea  ;;;
;;; copy-area ;;;
;;;;;;;;;;;;;;;;;

(define copy-area
  (let ((opcode 62)
	(lock (make-lock))
	(str  (make-string 28)))
    (number->card8/fill!  str 0 opcode)
    (number->card8/fill!  str 1 0)  ; unused
    (number->card16/fill! str 2 7)  ; requests length
    (lambda (connection source-drawable dest-drawable gc
	     source-x source-y dest-x dest-y width height)
      (with-lock lock
	(lambda ()
	  (number->card32/fill! str  4 source-drawable)
	  (number->card32/fill! str  8 dest-drawable)
	  (number->card32/fill! str 12 gc)
	  (number->int16/fill!  str 16 source-x)
	  (number->int16/fill!  str 18 source-y)
	  (number->int16/fill!  str 20 dest-x)
	  (number->int16/fill!  str 22 dest-y)
	  (number->card16/fill! str 24 width)
	  (number->card16/fill! str 26 height)
	  (send (connection-xserver connection) str))))))


;;;;;;;;;;;;;;;;
;;; CreateGC ;;;
;;;;;;;;;;;;;;;;

;;; see create-gc-request.scm


;;;;;;;;;;;;;;;;;;;;;
;;; CreatePixmap  ;;;
;;; create-pixmap ;;;
;;;;;;;;;;;;;;;;;;;;;

(define create-pixmap
  (let ((opcode 53)
	(lock (make-lock))
	(str (make-string 16)))
    (number->card8/fill!  str 0 opcode)
    (number->card16/fill! str 2 4)  ; request length
    (lambda (connection drawable depth width height)
      (let ((pixmap-id (new-id)))
	(with-lock lock
          (lambda ()
	    (number->card8/fill!    str 1 depth)
	    (number->pixmap/fill!   str 4 pixmap-id)
	    (number->drawable/fill! str 8 drawable)
	    (number->card16/fill!   str 12 width)
	    (number->card16/fill!   str 14 height)
	    (send (connection-xserver connection) str)))
	pixmap-id))))


;;;;;;;;;;;;;;;;;;;;
;;; CreateWindow ;;;
;;;;;;;;;;;;;;;;;;;;

;;; set create-window-request.scm


;;;;;;;;;;;;;;;;;
;;; GetImage  ;;;
;;; get-image ;;;
;;;;;;;;;;;;;;;;;

;;; not tested

(define get-image
  (let ((opcode 73)
	(lock (make-lock))
	(str (make-string 20)))
    (number->card8/fill!  str 0 opcode)
    (number->card16/fill! str 2 5)  ; request length
    (lambda (connection drawable x y width height plane-mask format)
      (let ((format (case format
		      ((xy-pixmap) 1)
		      ((z-pixmap)  2)
		      (else (error "bad format" format))))
	    (plane-mask (case plane-mask
			  ((all-planes) #xffffffff)
			  (else plane-mask))))
	(with-lock lock
	  (lambda ()
	    (number->card8/fill!  str  1 format)
	    (number->drawable/fill! str  4 drawable)
	    (number->int16/fill!  str  8 x)
	    (number->int16/fill!  str 10 y)
	    (number->card16/fill! str 12 width)
	    (number->card16/fill! str 14 height)
	    (number->card32/fill! str 16 plane-mask)
	    (send+read-reply (connection-xserver connection)
			     str opcode (* width height))))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; GetKeyboardMapping   ;;;
;;; get-keyboard-mapping ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define get-keyboard-mapping
  (let ((opcode 101)
	(lock (make-lock))
	(str (make-string 8)))
    (number->card8/fill!  str 0 opcode)
    (number->card8/fill!  str 1 0)  ; unused
    (number->card16/fill! str 2 2)  ; request length
    (number->card16/fill! str 6 0)  ; unused
    (lambda (connection first-keycode count)
      (with-lock lock
        (lambda ()
	  (number->keycode/fill! str 4 first-keycode)
	  (number->card8/fill!   str 5 count)
	  (send+read-reply (connection-xserver connection) str opcode))))))


;;;;;;;;;;;;;;;;;;;;;;;;
;;; GetScreenSaver   ;;;
;;; get-screen-saver ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

(define get-screen-saver
  (let ((opcode 108)
	(str (make-string 4)))
    (number->card8/fill!  str 0 opcode)
    (number->card8/fill!  str 1 0)  ; unused
    (number->card16/fill! str 2 1)  ; request length
    (lambda (connection)
      (send+read-reply (connection-xserver connection) str opcode))))


;;;;;;;;;;;;;;;;;;;
;;; ImageText8  ;;;
;;; image-text8 ;;;
;;;;;;;;;;;;;;;;;;;

(define image-text8
  (let ((opcode 76)
	(lock (make-lock))
	(str (make-string 16)))
    (number->card8/fill! str 0 opcode)
    (lambda (connection drawable gc x y string)
      (bind ((n p padded-str (pad-data string)))
	(let ((xserver (connection-xserver connection))
	      (n+p (+ n p)))
	  (when (> n+p 262128)  ; (+ n p) <= (* (- (expt 2 16) 4) 4) = 262128
	    (error "string too long" n))
	  (let ((padded-str8 (string->string8 padded-str))
		(request-length (+ 4 (quotient n+p 4))))
	    (with-lock lock
	      (lambda ()
		(number->card8/fill!    str  1 n)  ; string length
		(number->card16/fill!   str  2 request-length)
		(number->drawable/fill! str  4 drawable)
		(number->gcontext/fill! str  8 gc)
		(number->int16/fill!    str 12 x)
		(number->int16/fill!    str 14 y)
		(send xserver str padded-str8)))))))))


;;;;;;;;;;;;;;;;;;;;
;;; ImageText16  ;;;
;;; image-text16 ;;;
;;;;;;;;;;;;;;;;;;;;

(define low-image-text16
  (let ((opcode 77)
	(lock (make-lock))
	(str (make-string 16)))
    (number->card8/fill! str 0 opcode)
    (lambda (connection drawable gc x y string16)
      (bind ((n*2 p padded-string16 (pad-data string16)))
	(let ((xserver (connection-xserver connection))
	      (n*2+p (+ n*2 p)))
	  (when (> n*2+p 262128)  ; (+ (* n 2) p) <= (* (- (expt 2 16) 4) 4) = 262128
	    (error "string too long" n*2))
	  (let ((request-length (+ 4 (quotient n*2+p 4))))
	    (with-lock lock
	      (lambda ()
		(number->card8/fill!    str 1 n*2)  ; string length
		(number->card16/fill!   str 2 request-length)
		(number->drawable/fill! str 4 drawable)
		(number->gcontext/fill! str 8 gc)
		(number->int16/fill!    str 12 x)
		(number->int16/fill!    str 16 y)
		(send xserver str padded-string16)))))))))


;;;;;;;;;;;;;;;;;;;;;;;
;;; ListExtensions  ;;;
;;; list-extensions ;;;
;;;;;;;;;;;;;;;;;;;;;;;

(define list-extensions
  (let ((opcode 99)
	(str (make-string 4)))
    (number->card8/fill!  str 0 opcode)
    (number->card8/fill!  str 1 0)  ; unused
    (number->card16/fill! str 2 1)  ; request length
    (lambda (connection)
      (send+read-reply (connection-xserver connection) str opcode))))


;;;;;;;;;;;;;;;;;;
;;; MapWindow  ;;;
;;; map-window ;;;
;;;;;;;;;;;;;;;;;;

(define map-window
  (let ((opcode 8)
	(lock (make-lock))
	(str (make-string 8)))
    (number->card8/fill!  str 0 opcode)
    (number->card8/fill!  str 1 0)  ; unused
    (number->card16/fill! str 2 2)  ; request length
    (lambda (connection window)
      (with-lock lock
        (lambda ()
	  (number->window/fill! str 4 window)
	  (send (connection-xserver connection) str))))))


;;;;;;;;;;;;;;;;;;;;
;;; NoOperation  ;;;
;;; no-operation ;;;
;;;;;;;;;;;;;;;;;;;;

(define no-operation
  (let ((opcode 127)
	(str (make-string 4)))
    (number->card8/fill!  str 0 opcode)
    (number->card8/fill!  str 1 0)  ; unused
    (number->card16/fill! str 2 1)  ; request length
    (lambda (connection)
      (send (connection-xserver connection) str))))


;;;;;;;;;;;;;;;;;
;;; OpenFont  ;;;
;;; open-font ;;;
;;;;;;;;;;;;;;;;;

(define open-font
  (let ((opcode 45)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill!  str  0 opcode)
    (number->card8/fill!  str  1 0)  ; unused
    (number->card16/fill! str 10 0)  ; unused
    (lambda (connection name)
      (let ((xserver (connection-xserver connection))
	    (font-id (new-id)))
	(bind ((n p padded-name (pad-data name)))
	  (let ((request-length (+ 3 (quotient (+ n p) 4))))
	    (with-lock lock
              (lambda ()
		(number->card16/fill! str 2 request-length)
		(number->card32/fill! str 4 font-id)
		(number->card16/fill! str 8 n)
		(send xserver str padded-name)
		font-id))))))))


;;;;;;;;;;;;;;;;;;;;;
;;; PolyFillArc   ;;;
;;; poly-fill-arc ;;;
;;;;;;;;;;;;;;;;;;;;;

;;; arc = '(<x> <y> <width> <height> <angle1> <angle2>)
;;; arc = '#(<x> <y> <width> <height> <angle1> <angle2>)

(define poly-fill-arc
  (let ((opcode 71)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill! str 0 opcode)
    (number->card8/fill! str 1 0)  ; unused
    (lambda (connection drawable gc arcs)
      (let* ((xserver (connection-xserver connection))
	     (number-of-arcs (length arcs))
	     (arcs (arc-structures->arcs arcs))
	     (request-length (+ 3 (* 3 number-of-arcs))))
	(with-lock lock
          (lambda ()
	    (number->card16/fill!   str 2 request-length)
	    (number->drawable/fill! str 4 drawable)
	    (number->gcontext/fill! str 8 gc)
	    (send xserver str arcs)))))))
    

;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; PolyFillRectangle   ;;;
;;; poly-fill-rectangle ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; rectangle  = (<x> <y> <width> <height>)
;;; rectangle  = #(<x> <y> <width> <height>)

(define poly-fill-rectangle
  (let ((opcode 70)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill! str 0 opcode)
    (number->card8/fill! str 1 0)  ; unused
    (lambda (connection drawable gc rectangle-list)
      (let* ((xserver (connection-xserver connection))
	     (number-of-rectangles (length rectangle-list))
	     (rectangles (rectangle-structures->rectangles rectangle-list))
	     (request-length (+ 3 (* 2 number-of-rectangles))))
	(with-lock lock
          (lambda ()
	    (number->card16/fill! str 2 request-length)
	    (number->card32/fill! str 4 drawable)
	    (number->card32/fill! str 8 gc)
	    (send xserver str rectangles)))))))


;;;;;;;;;;;;;;;;;
;;; PolyLine  ;;;
;;; poly-line ;;;
;;;;;;;;;;;;;;;;;

;;; point = '(x . y)
;;; point = '(x y)
;;; point = '#(x y)

(define poly-line
  (let ((opcode 65)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill! str 0 opcode)
    (lambda (connection drawable gc coordinate-mode points)
      (let ((coordinate-mode (case coordinate-mode
			       ((origin)   0)
			       ((previous) 1)
			       (else (error "unrecognized value for coordinate-mode"
					    coordinate-mode))))
	    (points-list (point-structures->points points))
	    (request-length (+ 3 (length points))))
	(with-lock lock
          (lambda ()
	    (number->card8/fill!    str 1 coordinate-mode)
	    (number->card16/fill!   str 2 request-length)
	    (number->drawable/fill! str 4 drawable)
	    (number->gcontext/fill! str 8 gc)
	    (send (connection-xserver connection) str points-list)))))))


;;;;;;;;;;;;;;;;;;
;;; PolyPoint  ;;;
;;; poly-point ;;;
;;;;;;;;;;;;;;;;;;

(define poly-point
  (let ((opcode 64)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill! str 0 opcode)
    (lambda (connection drawable gc coordinate-mode points)
      (let* ((xserver (connection-xserver connection))
	     (points-n (length points))
	     (coordinate-mode (case coordinate-mode
				((origin)   0)
				((previous) 1)
				(else (error "unrecognized coordinate-mode" coordinate-mode))))
	     (points-list (point-structures->points points))
	     (request-length (+ 3 points-n)))
	(with-lock lock
	  (lambda ()
	    (number->card8/fill!    str 1 coordinate-mode)
	    (number->card16/fill!   str 2 request-length)
	    (number->drawable/fill! str 4 drawable)
	    (number->gcontext/fill! str 8 gc)
	    (send xserver str points-list)))))))


;;;;;;;;;;;;;;;;;
;;; PutImage  ;;;
;;; put-image ;;;
;;;;;;;;;;;;;;;;;

;;; not tested

(define put-image
  (let ((opcode 72)
	(lock (make-lock))
	(str (make-string 24)))
    (number->card8/fill!  str  0 opcode)
    (number->card16/fill! str 22 0)  ; unused
    (lambda (connection drawable gc depth width height x y left-pad format data)
      (bind ((n p padded-data (pad-data data)))
	(let ((format (case format
			((bitmap)    0)
			((xy-pixmap) 1)
			((z-pixmap)  2)
			(else (error "bad format given" format))))
	      (request-length (quotient (+ 6 n p) 4)))
	  (with-lock lock
            (lambda ()
	      (number->card8/fill!    str  1 format)
	      (number->card16/fill!   str  2 request-length)
	      (number->drawable/fill! str  4 drawable)
	      (number->gcontext/fill! str  8 gc)
	      (number->card16/fill!   str 12 width)
	      (number->card16/fill!   str 14 height)
	      (number->int16/fill!    str 16 x)
	      (number->int16/fill!    str 18 y)
	      (number->card8/fill!    str 20 left-pad)
	      (number->card8/fill!    str 21 depth)
	      (send (connection-xserver connection) str padded-data))))))))


;;;;;;;;;;;;;;;;;;
;;; QueryFont  ;;;
;;; query-font ;;;
;;;;;;;;;;;;;;;;;;

(define query-font
  (let ((opcode 47)
	(lock (make-lock))
	(str (make-string 8)))
    (number->card8/fill!  str 0 opcode)
    (number->card8/fill!  str 1 0)  ; unused
    (number->card16/fill! str 2 2)  ; request length
    (lambda (connection fontable)
      (let ((xserver (connection-xserver connection)))
	(with-lock lock
          (lambda ()
	    (number->fontable/fill! str 4 fontable)
	    (send+read-reply xserver str opcode)))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; QueryTextExtents   ;;;
;;; query-text-extents ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; a fontable can be either a font or a graphics context

(define query-text-extents
  (let ((opcode 48)
	(lock (make-lock))
	(str (make-string 8)))
    (number->card8/fill! str 0 opcode)
    (lambda (connection fontable string16)
      (bind ((n*2 p padded-string16 (pad-data string16)))
	(let ((xserver (connection-xserver connection))
	      (p=2?  (= p 2))
	      (n*2+p (+ n*2 p)))
	  ;; (+ (* 2 n) p) <= (* (- (expt 2 16) 2) 4) = 262136
	  (when (> n*2+p 262136)
	    (error "string16 too long" n*2))
	  (let ((request-length (+ 2 (quotient n*2+p 4)))
		(trailer (if p=2? (number->card16 0) "")))
	    (with-lock lock
              (lambda ()
		(boolean->bool/fill!    str 1 p=2?)  ; odd length?
		(number->card16/fill!   str 2 request-length)
		(number->fontable/fill! str 4 fontable)
		(send+read-reply xserver (list str padded-string16 trailer) opcode)))))))))


;;;;;;;;;;;;;;;;;;;;;;;;
;;; SetScreenSaver   ;;;
;;; set-screen-saver ;;;
;;;;;;;;;;;;;;;;;;;;;;;;

(define set-screen-saver
  (let ((opcode 107)
	(lock (make-lock))
	(str (make-string 12)))
    (number->card8/fill!  str  0 opcode)
    (number->card8/fill!  str  1 0)  ; unused
    (number->card16/fill! str  2 3)  ; request length
    (number->card16/fill! str 10 0)  ; unused
    (lambda (connection timeout interval prefer-blanking allow-exposures)
      (let ((timeout (cond ((eq? timeout 'default) -1)
			   ((and (integer? timeout) (positive? timeout)) timeout)
			   (else (error "timeout should be a positive integer or 'default"
					timeout))))
	    (interval (cond ((eq? interval 'default) -1)
			    ((and (integer? interval) (positive? interval)) interval)
			    (else (error "interval should be a positive integer or 'default"
					 interval))))
	    (prefer-blanking (case prefer-blanking
			       ((#f)      0)
			       ((#t)      1)
			       ((default) 2)
			       (else (error "prefer-blanking should be a boolean or 'default"
					    prefer-blanking))))
	    (allow-exposures (case allow-exposures
			       ((#f)      0)
			       ((#t)      1)
			       ((default) 2)
			       (else (error "allow-exposures should be a boolean or 'default"
					    allow-exposures)))))
	(with-lock lock
          (lambda ()
	    (number->int16/fill! str 4 timeout)
	    (number->int16/fill! str 6 interval)
	    (number->card8/fill! str 8 prefer-blanking)
	    (number->card8/fill! str 9 allow-exposures)
	    (send (connection-xserver connection) str)))))))
