;;; toys.scm


;;;;;;;;;;;;;;;;;;;
;;; Hello World ;;;
;;;;;;;;;;;;;;;;;;;

(define hello-world
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((parent (root-window connection))
	     (depth  (root-depth connection))
	     (white  (white-pixel connection))
	     (black  (black-pixel connection))
	     (message "Hello World!")
	     (window-width  200)
	     (window-height 100))
	 (let* ((window (create-window
			 connection parent  0 0  window-width window-height
			 1 depth 'copy-from-parent 'copy-from-parent
			 `((event-mask (exposure button-press key-press))
			   (background-pixel ,white))))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white)) ))
		(text-query (query-text-extents connection gc (string->string16 message))))
	   (map-window connection window)
	   (let loop ((event (read-event connection)))
	     (cond ((expose-event? event)
		    (let ((string-width (query-text-extents-reply:overall-width text-query))
			  (string-height
			   (+ (query-text-extents-reply:overall-ascent text-query)
			      (query-text-extents-reply:overall-descent text-query))))
		      (format #t "string-width: ~a, string-height ~a~%"
			      string-width string-height)
		      (image-text8 connection window gc
				   (- (quotient window-width 2)  (quotient string-width 2))
				   (+ (quotient window-height 2) (quotient string-height 2))
				   message))
		    (loop (read-event connection)))
		   ((or (key-press-event? event)
			(button-press-event? event)) #t)
		   (else (format #t "Huh?~%"))))
	   ))))))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Hello World with specified font ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define hello-world-w/font
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((parent (root-window connection))
	     (depth  (root-depth connection))
	     (white  (white-pixel connection))
	     (black  (black-pixel connection))
	     (message "Hello World!") ;"\033$(B2=@P$N5-21\033(B")
	     (font-name "lucidasans-italic-24") ;"k14")
	     (window-width  250)
	     (window-height 100))
	 (let* ((window (create-window
			 connection parent  0 0  window-width window-height
			 1 depth 'copy-from-parent 'copy-from-parent
			 `((event-mask (exposure button-press key-press))
			   (background-pixel ,white))))
		(font (open-font connection font-name))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white)
						   (font ,font)) ))
		(text-query (query-text-extents connection gc (string->string16 message))))
	   (map-window connection window)
	   (let loop ((event (read-event connection)))
	     (cond ((expose-event? event)
		    (let ((string-width
			   (query-text-extents-reply:overall-width text-query))
			  (string-height
			   (+ (query-text-extents-reply:overall-ascent text-query)
			      (query-text-extents-reply:overall-descent text-query))))
		      (image-text8 connection window gc
				   (- (quotient window-width 2)  (quotient string-width 2))
				   (+ (quotient window-height 2) (quotient string-height 2))
				   message))
		    (loop (read-event connection)))
		   ((or (key-press-event? event)
			(button-press-event? event)) #t)
		   (else (format #t "Huh?~%"))))
	   ))))))


;;;;;;;;;;;;;
;;; Shade ;;;
;;;;;;;;;;;;;

;;; Draw dark to blue shading in a window.

(define shade
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((parent (root-window connection))
	     (depth  (root-depth connection))
	     (white  (white-pixel connection))
	     (black  (black-pixel connection))
	     (colormap (default-colormap connection))
	     (window-width  100)
	     (window-height 100))
	 (let* ((window (create-window connection parent
				       0 0  window-width window-height
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure button-press key-press))
					 (background-pixel ,white))))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white))))
		(increment (quotient #xffff window-height)))
	   (map-window connection window)
	   (let loop ((event (read-event connection)))
	     (cond ((expose-event? event)
		    (do ((y 0 (+ y 1)))
			((> y window-height) #t)
		      (let* ((color-index (* y increment))
			     (color (alloc-color-reply:pixel
				     (alloc-color connection colormap 0 0 color-index))))
			(change-gc connection gc `((foreground ,color)))
			(poly-line connection window gc 'origin
				   (list (vector 0 y) (vector window-width y)))))
		    (loop (read-event connection)))
		   ((or (key-press-event? event)
			(button-press-event? event)) #t)
		   (else (format #t "Huh?~%"))))
	   ))))))


;;;;;;;;;;;;;;
;;; Bounce ;;;
;;;;;;;;;;;;;;

;;; bounce a ball
;;; Let's see if I remember how this all works.
;;; The ball will move up and down on the Y axis.
;;; Gravity accelerates it down at 9.8 m/s2.
;;; The ball will have a position, and a velocity.
;;; When the ball hits position 200, it reverses velocity.

(define exact-round
  (lambda (x)
    (let ((rx (round x)))
      (if (inexact? rx)
	  (inexact->exact rx)
	  rx))))

(define bounce
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((parent (root-window connection))
	     (depth  (root-depth connection))
	     (white  (white-pixel connection))
	     (black  (black-pixel connection))
	     (window-width  100)
	     (window-height 200))
	 (let* ((window (create-window
			 connection parent  0 0  window-width window-height
			 1 depth 'copy-from-parent 'copy-from-parent
			 `((event-mask (exposure button-press key-press))
			   (background-pixel ,white))))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white)
						   (graphics-exposures #f) ))))
	   (format #t "Map Window~%")
	   (map-window connection window)
	   (let loop ((event (read-event connection)))
	     (cond ((expose-event? event)
		    (let ((a 9) (dt 0.08) (r 40) (x 30) (circ (* 64 360)))
		      (let ((radius-bottom (- window-height r)))
			(let loop ((tp 0) (yp 10) (vp 0)
				   (t 0)  (y 10)  (v 0))
			  (cond ((and (= y yp radius-bottom) (< (abs v) 5)) #t)
				((> y radius-bottom)
				 (let ((scale (/ (- radius-bottom y) (- y yp))))
				   (loop tp yp vp
					 (+ tp (* scale (- t tp)))
					 radius-bottom
					 (* -0.9 (+ vp (* scale (- v vp)))))))
				(else
				 (clear-area connection window x yp r r #t)
				 (poly-fill-arc connection window gc `((,x ,y ,r ,r 0 ,circ)))
				 (loop t y v
				       (+ t dt)
				       (+ y (exact-round (* v dt)))
				       (+ v (* a dt)))))))))
		   ((or (key-press-event? event)
			(button-press-event? event)) #t)
		   (else (format #t "Huh?~%"))))
	   ))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Bounce with double buffering ;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(define bounce-db
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((parent (root-window connection))
	     (depth  (root-depth connection))
	     (white  (white-pixel connection))
	     (black  (black-pixel connection))
	     (window-width  100)
	     (window-height 200))
	 (let* ((window (create-window connection parent 0 0 window-width window-height
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure button-press))
					 (background-pixel ,white))))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white)
						   (graphics-exposures #f) )))
		(copy-gc (create-gc connection window `((foreground ,white)
							(background ,black)
							(graphics-exposures #f) )))
		(scratch-pixmap (create-pixmap connection window depth
					       window-width window-height))
		(clear-pixmap   (create-pixmap connection window depth
					       window-width window-height)))
	   (format #t "Map Window~%")
	   (map-window connection window)
	   (let loop ((event (read-event connection)))
	     (cond ((or (expose-event? event)
			(graphics-exposure-event? event))
		    (copy-area connection window clear-pixmap copy-gc
			       0 0 0 0 window-width window-height)
		    (let ((a 9) (dt 0.3) (r 40) (x 30) (circ (* 64 360)))
		      (let ((radius-bottom (- window-height r)))
			(let bounce ((tp 0) (yp 10) (vp 0)
				     (t 0)  (y 10)  (v 0))
			  (cond ((and (= y yp radius-bottom) (< (abs v) 5))
				 (format #t "Done~%")
				 (loop (read-event connection)))
				((> y radius-bottom)
				 (let ((scale (/ (- radius-bottom y) (- y yp))))
				   (bounce tp yp vp
					   (+ tp (* scale (- t tp)))
					   radius-bottom
					   (* -0.9 (+ vp (* scale (- v vp)))))))
				(else
				 (copy-area connection clear-pixmap scratch-pixmap copy-gc
					    0 0 0 0 window-width window-height)
				 (poly-fill-arc connection scratch-pixmap gc
						`((,x ,y ,r ,r 0 ,circ)))
				 (copy-area connection scratch-pixmap window copy-gc
					    0 0 0 0 window-width window-height)
				 (bounce t y v
					 (+ t dt)
					 (+ y (exact-round (* v dt)))
					 (+ v (* a dt)))))))))
		   ((button-press-event? event) #t)
		   (else ;(display-event event)
		    (loop (read-event connection)))))
	   ))))))


;;;;;;;;;;;;;;;;;;;;;;;
;;; Threaded Bounce ;;;
;;;;;;;;;;;;;;;;;;;;;;;

(define bounce-th
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((parent (root-window connection))
	     (depth  (root-depth connection))
	     (white  (white-pixel connection))
	     (black  (black-pixel connection))
	     (window-width  100)
	     (window-height 200))
	 (let* ((window (create-window connection parent 0 0 window-width window-height
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure button-press))
					 (background-pixel ,white))))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white)
						   (graphics-exposures #f) )))
		(copy-gc (create-gc connection window `((foreground ,white)
							(background ,black)
							(graphics-exposures #f) )))
		(scratch-pixmap (create-pixmap connection window depth
					       window-width window-height))
		(clear-pixmap   (create-pixmap connection window depth
					       window-width window-height))
		(bouncing? #f)
		(bouncing-lock (make-lock))
		(bounce
		 (lambda ()
		   (set! bouncing? #t)
		   (copy-area connection window clear-pixmap copy-gc
			      0 0 0 0 window-width window-height)
		   (let ((a 9) (dt 0.3) (r 40) (x 30) (circ (* 64 360)))
		     (let ((radius-bottom (- window-height r)))
		       (let loop ((tp 0) (yp 10) (vp 0)
				  (t 0)  (y 10)  (v 0))
			 (cond ((and (= y yp radius-bottom) (< (abs v) 5)) ; finished
				(set! bouncing? #f)
				(format #t "Done~%"))
			       ((> y radius-bottom)  ; correct its position
				(let ((scale (/ (- radius-bottom y) (- y yp))))
				  (loop tp yp vp
					(+ tp (* scale (- t tp)))
					radius-bottom
					(* -0.9 (+ vp (* scale (- v vp)))))))
			       (else
				(copy-area connection clear-pixmap scratch-pixmap copy-gc
					   0 0 0 0 window-width window-height)
				(poly-fill-arc connection scratch-pixmap gc
					       `((,x ,y ,r ,r 0 ,circ)))
				(copy-area connection scratch-pixmap window copy-gc
					   0 0 0 0 window-width window-height)
				(loop t y v
				      (+ t dt)
				      (+ y (exact-round (* v dt)))
				      (+ v (* a dt)))))))))))
	   (format #t "Map Window~%")
	   (map-window connection window)
	   (let* ((bouncing-thunk
		   (lambda () (with-lock bouncing-lock bounce)))
		  (connection-thunk
		   (lambda ()
		     (let loop ((event (read-event connection)))
		       (cond ((or (expose-event? event)
				  (graphics-exposure-event? event))
			      (if (not bouncing?)
				  (spawn bouncing-thunk))
			      (loop (read-event connection)))
			     ((button-press-event? event) #t)
			     (else (loop (read-event connection))))))))
	     (with-multitasking
	      (lambda ()
		(spawn connection-thunk))))
	   ))))))


;;;;;;;;;;;;;;;;;;;;
;;; color change ;;;
;;;;;;;;;;;;;;;;;;;;

;;; The enscribed circle is white when the pointer is outside it, red when inside it.

(define color-change
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((parent (root-window connection))
	     (depth  (root-depth connection))
	     (white  (white-pixel connection))
	     (black  (black-pixel connection))
	     (colormap (default-colormap connection))
	     (window-width  100)
	     (window-height 100))
	 (let* ((window (create-window
			 connection parent  0 0  window-width window-height
			 1 depth 'copy-from-parent 'copy-from-parent
			 `((event-mask (exposure button-press key-press
					enter-window leave-window))
			   (background-pixel ,white)
			   (colormap ,colormap))))
		(red (alloc-color-reply:pixel (alloc-color connection colormap #xffff 0 0)))
		(gc (create-gc connection window `((foreground ,black)
						   (background ,white)) ))
		(draw-circle
		 (lambda (color)
		   (change-gc connection gc `((foreground ,color)))
		   (poly-fill-arc connection window gc
				  '(#(0 0 100 100 0 23040))))))
	   (map-window connection window)
	   (do ((event (read-event connection) (read-event connection)))
	       ((or (key-press-event? event)
		    (button-press-event? event)) #t)
	     (cond ((enter-notify-event? event)
		    (draw-circle red))
		   ((leave-notify-event? event)
		    (draw-circle white))
		   ((expose-event? event) #t)
		   (else (display-event event))))
	   ))))))


;;;;;;;;;;;;;
;;; Mover ;;;
;;;;;;;;;;;;;

;;; (old data)
;; number->uint8:  12832 times
;; number->uint16:  6393
;; number->uint32:  1468
;; uint8->number:     40
;; uint16->number:    27
;; uint32->number:    39
;; send:             549
;; send+reply:         0

;;; poly-fill-rectangle produces 1440 bytes (* 180-iterations 8-bytes)

(define mover
  (lambda ()
    (with-connection
     (lambda (connection)
       (let ((parent (root-window connection))
	     (depth  (root-depth connection))
	     (white  (white-pixel connection))
	     (black  (black-pixel connection))
	     (window-width  200)
	     (window-height 30)
	     (size 20))
	 (let* ((window (create-window connection parent 0 0 window-width window-height
				       1 depth 'copy-from-parent 'copy-from-parent
				       `((event-mask (exposure button-press))
					 (background-pixel ,white) )))
		(gc (create-gc connection window
			       `((foreground ,black)
				 (background ,white)
				 (graphics-exposures #f) )))
		(clear-gc (create-gc connection window
				     `((foreground ,white)
				       (background ,black)
				       (graphics-exposures #f) )))
		(clear-pixmap (create-pixmap connection window depth size size))
		(draw-pixmap (create-pixmap connection window depth window-width window-height)))
	   (format #t "Map Window~%")
	   (map-window connection window)
	   (let loop ((event (read-event connection)))
	     (cond ((expose-event? event)
		    (copy-area connection window clear-pixmap clear-gc 0 0 0 0 size size)
		    (copy-area connection window draw-pixmap gc
			       0 0 0 0 window-width window-height)
		    (let* ((y (quotient (- window-height size) 2))
			   (rec (list (vector 'x y size size))))
		      (let draw ((old-x 0) (x 0))
			(if (>= x (- window-width size))
			    (loop (read-event connection))
			    (let ((before (time)))
			      (copy-area connection clear-pixmap draw-pixmap clear-gc
					 0 0 old-x y size size)
			      (vector-set! (car rec) 0 x)
			      (poly-fill-rectangle connection draw-pixmap gc rec)
			      (copy-area connection draw-pixmap window gc
					 0 0 0 0 window-width window-height)
			      (let ((t (- (time) before)))
				(when (> t 100)
				  (format #t "possible gc: ~a~%" t)))
			      (draw x (+ x 1)))))))
		   ((button-press-event? event) #t)
		   (else (loop (read-event connection)))))
	   ))))))
